# Copyright (C) 2023 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
#
"""Constants and helpers for this testing suite"""


import pathlib
from typing import Callable, Counter, Optional

from delivery.products import ProductMetadata

TEST_RESTORE_METADATA = ProductMetadata("EVLA", "17B-403", "restored_cms", None, None, "60468.6349189817")


def get_expected_files_and_dirs_for_restore(containing_dir: str) -> tuple[set[pathlib.Path], Counter[str]]:
    expected_dirs_to_file_counts = Counter(
        {
            f"{containing_dir}/17B-403.sb34574962.eb34577590_000.58035.544146898144.ms": 119,
            f"{containing_dir}/weblog_calibration": 1495,
            f"{containing_dir}/weblog_restore": 90,
        }
    )
    expected_files = {
        f"{containing_dir}/PPR_restore.xml",
        f"{containing_dir}/PPR_restore.xml",
        f"{containing_dir}/17B-403.sb34574962.eb34577590_000.58035.544146898144.ms.calapply.txt",
        f"{containing_dir}/casa_piperestorescript.py",
        f"{containing_dir}/casa_pipescript.py",
        f"{containing_dir}/casa_commands.log",
        f"{containing_dir}/PPR_calibration.xml",
        f"{containing_dir}/flux.csv",
        f"{containing_dir}/unknown.session_1.caltables.tgz",
    }
    expected_files = {pathlib.Path(f) for f in expected_files}
    return expected_files, expected_dirs_to_file_counts


def assert_restore_delivered_only_expected_files(
    actual_files: list[pathlib.Path],
    expected_files: set[pathlib.Path],
    expected_dirs_to_file_counts: Counter[str],
    actual_files_to_is_file: Optional[dict[pathlib.Path, bool]] = None,
):
    actual_is_file: Callable[[pathlib.Path], bool] = lambda f: f.is_file()
    if actual_files_to_is_file is not None:
        # actual_files' contents don't exist on disk, use given table to know which are files vs. directories
        actual_is_file = lambda f: actual_files_to_is_file[f]
    # did we get all the expected files?
    for filepath in expected_files:
        assert filepath in actual_files
    actual_dirs_to_file_counts = Counter()
    for filepath in actual_files:
        for dir, _ in expected_dirs_to_file_counts.items():
            if str(filepath).startswith(f"{dir}/") and actual_is_file(filepath):
                actual_dirs_to_file_counts[dir] += 1
    assert actual_dirs_to_file_counts == expected_dirs_to_file_counts

    # did we not get any unexpected files?
    for filepath in actual_files:
        if filepath.is_file():
            assert filepath in expected_files or any(
                str(filepath).startswith(expected_dir) for expected_dir in expected_dirs_to_file_counts.keys()
            )
